﻿Imports System
Imports System.ComponentModel
Imports System.ComponentModel.Design.Serialization
Imports System.Reflection
Imports System.Globalization
Imports System.Runtime.Serialization
Imports System.Security.Permissions

Imports Infragistics.Shared
Imports Infragistics.Shared.Serialization
Imports Infragistics.Win
Imports Infragistics.Win.UltraWinToolbars
Namespace ToolProvider

#Region " CustomButtonTool Class "
    'CustomButtonTool Class
    <TypeConverter(GetType(CustomButtonTool.CustomButtonToolConverter)), _
    Serializable()> _
    Public Class CustomButtonTool
        Inherits ButtonTool
        Implements INABiCustomProperty

#Region " Member Variables "
        'Hold property settings in temporary member variable during de-serialization. 
        'When our Initialize method is called at the end of de-serialization, move the
        'values to the actual ExtraSharedProps member which will be fully 'hooked up' by then.
        Private m_serializedProps As CustomButtonTool.CustomButtonToolExtraSharedProps = Nothing

#End Region   'Member Variables

#Region " Constructor "

#Region " Standard Constructor "

        Public Sub New(ByVal key As String)
            MyBase.New(key)
        End Sub


#End Region    'Standard Constructor

#Region " De-serialization Constructor "
        'Constructor used for de-serialization
        '2016-01-19 이재영 수정 : NABi Property 추가(NABiGropupKeyProp, NABiGroupNameProp, NABiTabIndexProp)
        Protected Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            MyBase.New(info, context)
            Dim entry As SerializationEntry
            For Each entry In info
                Select Case (entry.Name)
                    Case "NABiGroupKeyProp"
                        Me.SerializedProps.m_GroupKey = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupKey))
                    Case "NABiGroupNameProp"
                        Me.SerializedProps.m_GroupName = CStr(Utils.DeserializeProperty(entry, GetType(String), Me.SerializedProps.m_GroupName))
                    Case "NABiTabIndexProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_TabIndex))
                    Case "NABiButtonTypeProp"
                        Me.SerializedProps.m_TabIndex = CStr(Utils.DeserializeProperty(entry, GetType(Integer), Me.SerializedProps.m_ButtonType))
                End Select
            Next
        End Sub

#End Region    'De-serialization Constructor

#End Region   'Constructor

#Region " Constants "

        '2016-01-19 이재영 추가 
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPKEYPROP As String = "KeyValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABIGROUPNAMEPROP As String = "NameValue"
        Friend Shared ReadOnly DEFAULTVALUE_NABITABINDEXPROP As Integer = 0
        Friend Shared ReadOnly DEFAULTVALUE_NABIBUTTONTYPEPROP As Integer = 0

#End Region   'Constants

#Region " Base Class Overrides "

#Region "Clone"
        ' <summary>
        ' Returns a cloned copy of the tool.
        ' </summary>
        ' <param name="cloneNewInstance">If true, returns a clone of the tool that can serve as an instance of the tool, sharing the same SharedProps object.  If false, returns a tool that can be used as a new tool, with a clone of the original SharedProps object.</param>
        ' <returns></returns>
        Protected Overrides Function Clone(ByVal cloneNewInstance As Boolean) As Infragistics.Win.UltraWinToolbars.ToolBase
            Dim tool As CustomButtonTool = New CustomButtonTool(Me.Key)
            tool.InitializeFrom(Me, cloneNewInstance)
            Return tool
        End Function
#End Region 'Clone

#Region " GetObjectData "
        'Called from our base class when ISerializable.GetObjectData is called.  Serialize all custom property data here.
        '<param name="info"></param>
        '<param name="context"></param>
        <SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter:=True)> _
        Protected Overrides Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext)
            'Call the base implementation
            MyBase.GetObjectData(info, context)

            'Serialize our CustomProp property if necessary.
            If (Me.ShouldSerializeCustomProp()) Then
                '2016-01-19 이재영 추가
                Utils.SerializeProperty(info, "NABiGroupKeyProp", Me.NABiGroupKeyProp)
                Utils.SerializeProperty(info, "NABiGroupNameProp", Me.NABiGroupNameProp)
                Utils.SerializeProperty(info, "NABiTabIndexProp", Me.NABiTabIndexProp)
                Utils.SerializeProperty(info, "NABiButtonTypeProp", Me.NABiButtonTypeProp)
            End If
        End Sub

#End Region 'GetObjectData

#Region " Initialize "
        'Internal inherited method for initializing the tool when de-serialization completes.
        'Automatically called by toolbars manager.
        Protected Overrides Sub Initialize(ByVal parentCollection As ToolsCollectionBase)
            'Let the base do its processing.
            MyBase.Initialize(parentCollection)

            'Use the temporary member variable to set the actual ExtraSharedProps properties.
            'We need to wait until after deserialization is totally complete to ensure that
            'our ExtraSharedProps object is properly setup.
            If Not Me.m_serializedProps Is Nothing Then
                '2016-01-19 이재영 추가
                Me.ExtraSharedProps.m_GroupKey = Me.SerializedProps.m_GroupKey
                Me.ExtraSharedProps.m_GroupName = Me.SerializedProps.m_GroupName
                Me.ExtraSharedProps.m_TabIndex = Me.SerializedProps.m_TabIndex
                Me.ExtraSharedProps.m_ButtonType = Me.SerializedProps.m_ButtonType

                Me.m_serializedProps = Nothing
            End If

        End Sub

#End Region 'Initialize

#End Region   'Base Class Overrides

#Region " Properties "

#Region " Public Properties "

#Region " CustomProp "

        '2016-01-21 이재영 추가 : Button Type Property
        Public Property NABiButtonTypeProp() As ENUM_BUTTONTYPE Implements INABiCustomProperty.NABiButtonTypeProp
            Get
                Return Me.ExtraSharedProps.m_ButtonType
            End Get
            Set(value As ENUM_BUTTONTYPE)
                Me.ExtraSharedProps.m_ButtonType = value
            End Set
        End Property

        ' This is a custom property in a custom tool.
        '2016-01-19 이재영 추가 : Group Key Property
        Public Property NABiGroupKeyProp() As String Implements INABiCustomProperty.NABiGroupKeyProp
            Get
                Return Me.ExtraSharedProps.m_GroupKey
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupKey = Value
            End Set
        End Property
        '2016-01-19 이재영 추가 : Group Name Property
        Public Property NABiGroupNameProp() As String Implements INABiCustomProperty.NABiGroupNameProp
            'Delegate the get/sets for the property to our ExtraSharedProps object
            Get
                Return Me.ExtraSharedProps.m_GroupName
            End Get
            Set(ByVal Value As String)
                Me.ExtraSharedProps.m_GroupName = Value
            End Set
        End Property

        '2016-01-19 이재영 추가 : Tab Index Property
        Public Property NABiTabIndexProp() As ENUM_TAB_LIST Implements INABiCustomProperty.NABiTabIndexProp
            Get
                Return Me.ExtraSharedProps.m_TabIndex
            End Get
            Set(value As ENUM_TAB_LIST)
                Me.ExtraSharedProps.m_TabIndex = value
            End Set
        End Property

        'Returns a Boolean value that determines whether the CustomProp property is set to its default value.
        'Checked during serialization to determine if the property should be serialized.
        Protected Friend Overridable Function ShouldSerializeCustomProp() As Boolean
            'We only want to serialize the property if this tool is the Root tool and the 
            'property is not set to its default value.
            Return (Me.IsRootTool AndAlso Me.NABiGroupKeyProp <> DEFAULTVALUE_NABIGROUPKEYPROP AndAlso Me.NABiTabIndexProp <> DEFAULTVALUE_NABITABINDEXPROP)

        End Function

#End Region     'CustomProp

#End Region    'Public Properties

#Region " Private Properties "

#Region " SerializedProps "
        ' Used temporarily during de-serialization to hold or ExtraSharedProps values.
        Private ReadOnly Property SerializedProps() As CustomButtonTool.CustomButtonToolExtraSharedProps
            Get
                If (Me.m_serializedProps Is Nothing) Then
                    Me.m_serializedProps = New CustomButtonTool.CustomButtonToolExtraSharedProps(Me)
                End If
                Return Me.m_serializedProps
            End Get
        End Property

#End Region     'SerializedProps

#End Region    'Private Properties

#End Region   'Properties

#Region " Methods "
#Region " VerifyCanBeModified "
        Friend Sub VerifyBaseToolCanBeModified()
            MyBase.VerifyCanBeModified()
        End Sub
#End Region
#End Region

#Region " CustomButtonTool Type Converter "
        'Type converter for the CustomButtonTool class.  This type converter is necessary to ensure that
        'code generated in the form for creation of the tool is referencing the proper type.
        Public Class CustomButtonToolConverter
            Inherits ExpandableObjectConverter
            'CanConvertTo method.
            '<param name="context"></param>
            '<param name="destinationType"></param>
            '<returns></returns>
            Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
                If destinationType Is GetType(InstanceDescriptor) Then
                    Return True
                End If

                'Always call the base to see if it can perform the conversion.
                Return MyBase.CanConvertTo(context, destinationType)
            End Function

            'ConvertTo Method
            '<param name="context"></param>
            '<param name="culture"></param>
            '<param name="value"></param>
            '<param name="destinationType"></param>
            Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
                If destinationType Is GetType(InstanceDescriptor) Then
                    Dim ci As ConstructorInfo = GetType(CustomButtonTool).GetConstructor(New Type() {GetType(String)})
                    Dim buttonTool As CustomButtonTool = CType(value, CustomButtonTool)

                    Return New InstanceDescriptor(ci, New Object() {buttonTool.Key}, False)
                End If

                'Always call base, even if you can't convert.
                Return MyBase.ConvertTo(context, culture, value, destinationType)
            End Function
        End Class

#End Region   'CustomButtonTool Type Converter

#Region " ExtraSharedProps helper class, methods & properties "

#Region " ExtraSharedPropsCount property "
        'Returns the number of extra shared properties required by this class and all of its base classes.
        <Browsable(False), _
        DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Protected Overrides ReadOnly Property ExtraSharedPropsCount() As Integer
            Get
                Return MyBase.ExtraSharedPropsCount + 1
            End Get
        End Property

#End Region    'ExtraSharedPropsCount property

#Region " ExtraSharedProps property "
        'Returns our private ExtraSharedProps object.  All property gets/sets in the owning tool will
        'be delegated to similary named properties on the private ExtraSharedProps object.
        Private ReadOnly Property ExtraSharedProps() As CustomButtonToolExtraSharedProps
            Get
                If Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount) Is Nothing Then
                    'Create a new custom extra shared props object.
                    Dim m_ExtraSharedProps As CustomButtonToolExtraSharedProps = New CustomButtonToolExtraSharedProps(Me)

                    'Save our custom extra shared props object in SharedProps.
                    Me.SetExtraSharedProps(m_ExtraSharedProps, MyBase.ExtraSharedPropsCount)
                End If

                'Return our custom extra shared props object.
                Return CType(Me.GetExtraSharedProps(MyBase.ExtraSharedPropsCount), CustomButtonToolExtraSharedProps)
            End Get
        End Property

#End Region    'ExtraSharedProps property

#Region " ExtraSharedProps helper class "
        'CustomButtonToolExtraSharedProps helper class.  The ExtraSharedProps object allow us to provide 
        'properties whose settings are shared across all instances of a root tool just as the SharedProps
        'object does for each tool with the built-in shared tool proprties. The owning tool will delegate
        'all property gets/sets to similarly named properties on the ExtraSharedProps object.
        Private Class CustomButtonToolExtraSharedProps
            Inherits SubObjectBase
#Region " Member Variables "

            'The owning tool for which we are providing extra shared properties.
            Private owningTool As CustomButtonTool = Nothing

            'Member variables to hold values for each 'extra shared property'. 
            '2016-01-19 이재영 : Group Key를 받아오기 위한 변수
            Friend m_GroupKey As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPKEYPROP
            '2016-01-19 이재영 : Group Name을 받아오기 위한 변수
            Friend m_GroupName As String = CustomButtonTool.DEFAULTVALUE_NABIGROUPNAMEPROP
            '2016-01-19 이재영 : Tab Index를 받아오기 위한 변수
            Friend m_TabIndex As Integer = CustomButtonTool.DEFAULTVALUE_NABITABINDEXPROP
            '2016-01-21 이재영 : Button type을 받아오기 위한 변수
            Friend m_ButtonType As Integer = ENUM_BUTTONTYPE.NONE

#End Region    'Member Variables

#Region " Constructor "
            'Constructor
            Friend Sub New(ByVal owningTool As CustomButtonTool)
                'Initialize member variables.
                Me.owningTool = owningTool
            End Sub

#End Region    'Constructor

#Region " Properties "

#Region " CustomProp "
            ' This property supports the same-named property on our owning tool.
            Friend Property GroupKey() As String
                Get
                    Return Me.m_GroupKey
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupKey) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupKey = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            Friend Property GroupName() As String
                Get
                    Return Me.m_GroupName
                End Get
                Set(ByVal Value As String)
                    If (Value <> Me.m_GroupName) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.GroupName = Value

                        'Notify listeners of property change.
                        'me.owningTool.CallNotifyPropChange(PropertyIds.CustomProp)
                    End If
                End Set
            End Property

            '2016-01-19 이재영 추가
            Friend Property NABiTabIndexProp() As ENUM_TAB_LIST
                Get
                    Return Me.m_TabIndex
                End Get
                Set(value As ENUM_TAB_LIST)
                    If (value <> Me.m_TabIndex) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiTabIndexProp = value
                    End If
                End Set
            End Property

            '2016-01-21 이재영 추가
            Friend Property NABiButtonTypeProp() As ENUM_BUTTONTYPE
                Get
                    Return Me.m_ButtonType
                End Get
                Set(value As ENUM_BUTTONTYPE)
                    If (value <> Me.m_ButtonType) Then
                        'Verify that the tool can be modified
                        Me.owningTool.VerifyBaseToolCanBeModified()

                        'Save the value.
                        Me.NABiButtonTypeProp = value
                    End If
                End Set
            End Property

#End Region     'CustomProp

#End Region    'Properties
        End Class

#End Region    'ExtraSharedProps helper class

#End Region   'ExtraSharedProps helper class, methods & properties

    End Class

#End Region  'CustomButtonTool Class

    Public Interface INABiCustomProperty
        Property NABiGroupKeyProp() As String
        Property NABiGroupNameProp() As String
        Property NABiTabIndexProp() As ENUM_TAB_LIST
        Property NABiButtonTypeProp() As ENUM_BUTTONTYPE
    End Interface

End Namespace


